package com.aliyun.vodplayer.demo.activity.advance;

import android.Manifest;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

import com.aliyun.vodplayer.demo.utils.OnClickListener;
import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.Contants;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.UserStatusInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.AppEnterRequest;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.GetUserStatusRequest;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.utils.BaseRequest;

import java.lang.ref.WeakReference;
import java.util.UUID;

public class DemeLiveAnswerUrlActivity extends BaseAppCompatActivity {

    private EditText urlEditText;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_deme_live_answer_url);

        urlEditText = (EditText) findViewById(R.id.live_url);

        findViewById(R.id.start_answer).setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }

                int storagePermissionRet =
                        ContextCompat.checkSelfPermission(DemeLiveAnswerUrlActivity.this.getApplicationContext(),
                                Manifest.permission.READ_PHONE_STATE);
                if (storagePermissionRet != PackageManager.PERMISSION_GRANTED) {
                    requestPermission();
                    return;
                }

                String url = urlEditText.getText().toString().trim();
                if (TextUtils.isEmpty(url)) {
                    Toast.makeText(DemeLiveAnswerUrlActivity.this.getApplicationContext(), R.string.input_url, Toast.LENGTH_SHORT).show();
                    return;
                } else {
                    Contants.LIVE_URL = url.trim();
                    Contants.USER_ID = getUuid();
                    if (inRequest) {
                        //防止重复点击
                    } else {
                        goToAnswer();
                    }
                }
            }
        });
    }

    private int STORAGE_REQUEST_CODE = 100;

    private void requestPermission() {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.READ_PHONE_STATE)
                != PackageManager.PERMISSION_GRANTED) {
            // 向用户解释为什么需要这个权限
            new AlertDialog.Builder(this)
                    .setMessage(R.string.require_permision_msg)
                    .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            //申请权限
                            ActivityCompat.requestPermissions(DemeLiveAnswerUrlActivity.this,
                                    new String[]{Manifest.permission.READ_PHONE_STATE}, STORAGE_REQUEST_CODE);
                        }
                    })
                    .show();

        }
    }

    private void goToAnswer() {
        AppEnterRequest.EnterParam param = new AppEnterRequest.EnterParam();
        param.userId = Contants.USER_ID;
        param.liveId = Contants.getLiveId();

        AppEnterRequest enterRequest = new AppEnterRequest(this, param, new AppEnterRequestListener(this));
        inRequest = true;
        enterRequest.get();
    }

    private static class AppEnterRequestListener implements BaseRequest.OnRequestListener {


        private WeakReference<DemeLiveAnswerUrlActivity> activityWeakReference;

        public AppEnterRequestListener(DemeLiveAnswerUrlActivity activity) {
            activityWeakReference = new WeakReference<DemeLiveAnswerUrlActivity>(activity);
        }

        @Override
        public void onSuccess(Object requestInfo, String requestId) {
            DemeLiveAnswerUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                //进入直播间成功
                activity.requestMyStatus();
            }
        }

        @Override
        public void onFail(int code, String msg, String requestId) {
            DemeLiveAnswerUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                //进入直播间失败
                activity.enterFail();
            }

        }
    }

    private void enterFail() {
        inRequest = false;
        Toast.makeText(DemeLiveAnswerUrlActivity.this.getApplicationContext(), R.string.live_answer_enter_fail, Toast.LENGTH_LONG).show();
    }


    private void requestMyStatus() {

        GetUserStatusRequest.UserStatusParam param = new GetUserStatusRequest.UserStatusParam();
        param.liveId = Contants.getLiveId();
        param.userId = Contants.USER_ID.trim();

        GetUserStatusRequest request = new GetUserStatusRequest(this, param, new UserStatusRequestListener(this));
        request.get();

    }


    private static class UserStatusRequestListener implements BaseRequest.OnRequestListener<UserStatusInfo> {

        private WeakReference<DemeLiveAnswerUrlActivity> activityWeakReference;

        public UserStatusRequestListener(DemeLiveAnswerUrlActivity activity) {
            activityWeakReference = new WeakReference<DemeLiveAnswerUrlActivity>(activity);
        }

        @Override
        public void onSuccess(UserStatusInfo requestInfo, String requestId) {
            DemeLiveAnswerUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.goToLiveActivity(requestInfo.isSurvivor);
            }
        }

        @Override
        public void onFail(int code, String msg, String requestId) {
            DemeLiveAnswerUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                //进入直播间失败
                activity.goToLiveActivity(false);
            }
        }
    }


    private void goToLiveActivity(boolean isSurvivor) {
        Contants.SERVIOR = isSurvivor;

        Intent intent = new Intent();
        intent.setClass(this, DemoLiveAnswerActivity.class);
        startActivity(intent);

        inRequest = false;

    }


    private boolean inRequest = false;


    private String getUuid() {
        final TelephonyManager tm = (TelephonyManager) getApplicationContext().getSystemService(Context.TELEPHONY_SERVICE);

        final String tmDevice, tmSerial, androidId;
        tmDevice = "" + tm.getDeviceId();
        tmSerial = "" + tm.getSimSerialNumber();
        androidId = "" + android.provider.Settings.Secure.getString(getContentResolver(), android.provider.Settings.Secure.ANDROID_ID);

        UUID deviceUuid = new UUID(androidId.hashCode(), ((long) tmDevice.hashCode() << 32) | tmSerial.hashCode());
        String uniqueId = deviceUuid.toString();
        return uniqueId;

    }
}
