package com.aliyun.vodplayer.demo.activity.advance;

import android.Manifest;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.IdRes;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import com.alivc.player.AliyunErrorCode;
import com.alivc.player.VcPlayerLog;
import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.demo.utils.Formatter;
import com.aliyun.vodplayer.demo.utils.OnClickListener;
import com.aliyun.vodplayer.media.AliyunLocalSource;
import com.aliyun.vodplayer.media.AliyunVodPlayer;
import com.aliyun.vodplayer.media.IAliyunVodPlayer;
import com.aliyun.vodplayerview.utils.ScreenUtils;

import java.lang.ref.WeakReference;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import static com.aliyun.vodplayer.media.IAliyunVodPlayer.PlayerState.Completed;
import static com.aliyun.vodplayer.media.IAliyunVodPlayer.PlayerState.Idle;
import static com.aliyun.vodplayer.media.IAliyunVodPlayer.PlayerState.Stopped;

public class DemoChangeUrlActivity extends BaseAppCompatActivity {


    private static final String TAG = DemoChangeUrlActivity.class.getSimpleName();

    private SimpleDateFormat format = new SimpleDateFormat("HH:mm:ss.SS");

    private SurfaceView surfaceView;

    private Button playBtn;
    private Button pauseBtn;
    private Button replayBtn;
    private Button stopBtn;


    private TextView positionTxt;
    private TextView durationTxt;
    private SeekBar progressBar;


    private RadioButton videoBtn1;
    private RadioButton videoBtn2;
    private RadioButton videoBtn3;
    private RadioButton videoBtn4;
    private RadioButton videoBtn5;

    private AliyunLocalSource videoSource1;
    private AliyunLocalSource videoSource2;
    private AliyunLocalSource videoSource3;
    private AliyunLocalSource videoSource4;
    private AliyunLocalSource videoSource5;

    private List<String> logStrs = new ArrayList<>();

    private AliyunVodPlayer aliyunVodPlayer;
    private boolean replay = false;
    private boolean inSeek = false;
    private boolean isCompleted = false;

    private AliyunLocalSource mLocalSource = null;

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_log, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.log) {

            LayoutInflater inflater = LayoutInflater.from(this);
            View view = inflater.inflate(R.layout.view_log, null);

            TextView textview = (TextView) view.findViewById(R.id.log);
            if (aliyunVodPlayer != null) {
                for (String log : logStrs) {
                    textview.append("     " + log + "\n");
                }
            }
            AlertDialog.Builder alertDialog = new AlertDialog.Builder(this);
            alertDialog.setTitle(R.string.player_log);
            alertDialog.setView(view);
            alertDialog.setPositiveButton(R.string.ok, null);
            AlertDialog alert = alertDialog.create();
            alert.show();
        }
        return true;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        setContentView(R.layout.activity_demo_change_url);
        surfaceView = (SurfaceView) findViewById(R.id.surfaceView);

        playBtn = (Button) findViewById(R.id.play);
        stopBtn = (Button) findViewById(R.id.stop);
        pauseBtn = (Button) findViewById(R.id.pause);
        replayBtn = (Button) findViewById(R.id.replay);

        videoBtn1 = (RadioButton) findViewById(R.id.video1);
        videoBtn2 = (RadioButton) findViewById(R.id.video2);
        videoBtn3 = (RadioButton) findViewById(R.id.video3);
        videoBtn4 = (RadioButton) findViewById(R.id.video4);
        videoBtn5 = (RadioButton) findViewById(R.id.video5);

        RadioGroup videoGroup = (RadioGroup) findViewById(R.id.video_group);
        videoGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, @IdRes int checkedId) {
                if (checkedId == R.id.video1) {
                    mLocalSource = videoSource1;
                } else if (checkedId == R.id.video2) {
                    mLocalSource = videoSource2;
                } else if (checkedId == R.id.video3) {
                    mLocalSource = videoSource3;
                } else if (checkedId == R.id.video4) {
                    mLocalSource = videoSource4;
                } else if (checkedId == R.id.video5) {
                    mLocalSource = videoSource5;
                }

                //切换视频源播放：先stop，再prepare。
                isCompleted = false;
                inSeek = false;
                aliyunVodPlayer.stop();
                aliyunVodPlayer.prepareAsync(mLocalSource);
                showVideoProgressInfo();
            }
        });

        positionTxt = (TextView) findViewById(R.id.currentPosition);
        durationTxt = (TextView) findViewById(R.id.totalDuration);
        progressBar = (SeekBar) findViewById(R.id.progress);

        playBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }
                mPlayerState = aliyunVodPlayer.getPlayerState();
                if (mPlayerState == Idle || mPlayerState == Stopped ||
                        mPlayerState == Completed) {
                    if (mLocalSource != null) {
                        aliyunVodPlayer.prepareAsync(mLocalSource);
                    }
                } else {
                    inSeek = false;
                    aliyunVodPlayer.start();
                    pauseBtn.setText(R.string.pause_button);
                }

                logStrs.add(format.format(new Date()) + getString(R.string.log_strart_play));
            }
        });
        stopBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }
                if (aliyunVodPlayer != null) {
                    aliyunVodPlayer.stop();
                }
            }
        });

        pauseBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }
                mPlayerState = aliyunVodPlayer.getPlayerState();
                if (mPlayerState == IAliyunVodPlayer.PlayerState.Started) {
                    aliyunVodPlayer.pause();
                    pauseBtn.setText(R.string.resume_button);
                } else if (mPlayerState == IAliyunVodPlayer.PlayerState.Paused) {
                    aliyunVodPlayer.resume();
                    pauseBtn.setText(R.string.pause_button);
                }
            }
        });
        replayBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }

                isCompleted = false;
                inSeek = false;
                aliyunVodPlayer.replay();
                showVideoProgressInfo();

            }
        });


        progressBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {

            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
                if (aliyunVodPlayer != null) {
                    aliyunVodPlayer.seekTo(seekBar.getProgress());
                    logStrs.add(format.format(new Date()) + getString(R.string.log_seek_start));
                    if (isCompleted) {
                        //播放完成了，不能seek了
                        inSeek = false;
                    } else {
                        inSeek = true;
                    }
                }
            }
        });

        surfaceView.getHolder().addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                Log.d(TAG, "surfaceCreated");
                aliyunVodPlayer.setDisplay(holder);
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
                Log.d(TAG, "surfaceChanged");
                aliyunVodPlayer.surfaceChanged();
            }

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {
                Log.d(TAG, "surfaceDestroyed");
            }
        });

        initVodPlayer();
        initLocalSource();
        setPlaySource();
        logStrs.add(format.format(new Date()) + getString(R.string.log_start_get_data));

        if (mLocalSource != null) {
            aliyunVodPlayer.prepareAsync(mLocalSource);
        }

    }

    private void initLocalSource() {
        AliyunLocalSource.AliyunLocalSourceBuilder asb = new AliyunLocalSource.AliyunLocalSourceBuilder();
        asb.setSource("http://player.alicdn.com/video/aliyunmedia.mp4");
        asb.setTitle("阿里视频云介绍");
        videoBtn1.setText("阿里视频云介绍");
        videoSource1 = asb.build();

        asb.setSource("http://livetest.aliyunlive.com/9725750fa11648eba3426f65920ccecd/bce6da0b4afd43bb9fd06af269816b82-4b6ffae84f2e1d243955ecaedcf11a3e.m3u8");
        asb.setTitle("奥运会");
        videoBtn2.setText("奥运会");
        videoSource2 = asb.build();

        asb.setSource("http://livetest.aliyunlive.com/af6b16749fb44f2cb26c7c9b7cf3e3b2/3d8d93f8791c4e019388fd57e7dc9efb-4b6ffae84f2e1d243955ecaedcf11a3e.m3u8");
        asb.setTitle("多媒体");
        videoBtn3.setText("多媒体");
        videoSource3 = asb.build();

        asb.setSource("http://saas-video-qp.qupaicloud.com/299B3F9B-15BE76DF272-1767-9096-266-17559/8b8c03c80c2f43bf903a76621a6008d8.m3u8");
        asb.setTitle("MV素材演示");
        videoBtn4.setText("MV素材演示");
        videoSource4 = asb.build();

        asb.setSource("http://saas-video-qp.qupaicloud.com/4b5561fcda2d4825b2285a48e3b25389/1071bc468d85488bb98eef1867a98fca-699a80a29a15c59c9d0868389f86cc3f.mp4");
        asb.setTitle("MV效果演示");
        videoBtn5.setText("MV效果演示");
        videoSource5 = asb.build();
    }

    private void initVodPlayer() {
        aliyunVodPlayer = new AliyunVodPlayer(this);
        aliyunVodPlayer.setOnCircleStartListener(new MyCircleStartListener(this));
        aliyunVodPlayer.setOnPreparedListener(new MyPrepareListener(this));
        aliyunVodPlayer.setOnFirstFrameStartListener(new MyFirstFrameListener(this));
        aliyunVodPlayer.setOnErrorListener(new MyErrorListener(this));
        aliyunVodPlayer.setOnCompletionListener(new MyCompletionListener(this));
        aliyunVodPlayer.setOnSeekCompleteListener(new MySeekCompleteListener(this));
        aliyunVodPlayer.setOnStoppedListner(new MyStoppedListener(this));
        aliyunVodPlayer.setOnChangeQualityListener(new MyChangeQualityListener(this));
        aliyunVodPlayer.setReferer("http://aliyun.com");
        aliyunVodPlayer.enableNativeLog();
    }

    private static class MyCircleStartListener implements IAliyunVodPlayer.OnCircleStartListener {
        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MyCircleStartListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }

        @Override
        public void onCircleStart() {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onCircleStart();
            }
        }
    }

    private void onCircleStart() {
        //循环播放开始
    }

    private static class MyPrepareListener implements IAliyunVodPlayer.OnPreparedListener {

        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MyPrepareListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }

        @Override
        public void onPrepared() {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onPrepared();
            }
        }
    }

    private void onPrepared() {
        Toast.makeText(DemoChangeUrlActivity.this.getApplicationContext(), R.string.toast_prepare_success, Toast.LENGTH_SHORT).show();
        logStrs.add(format.format(new Date()) + getString(R.string.log_prepare_success));

        aliyunVodPlayer.start();
        pauseBtn.setText(R.string.pause_button);
        logStrs.add(format.format(new Date()) + getString(R.string.log_strart_play));
    }


    private static class MyFirstFrameListener implements IAliyunVodPlayer.OnFirstFrameStartListener {

        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MyFirstFrameListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }

        @Override
        public void onFirstFrameStart() {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onFirstFrameStart();
            }
        }
    }

    private void onFirstFrameStart() {

        //首帧显示之后再开始更新进度信息。
        inSeek = false;
        showVideoProgressInfo();


        Map<String, String> debugInfo = aliyunVodPlayer.getAllDebugInfo();
        long createPts = 0;
        if (debugInfo.get("create_player") != null) {
            String time = debugInfo.get("create_player");
            createPts = (long) Double.parseDouble(time);
            logStrs.add(format.format(new Date(createPts)) + getString(R.string.log_player_create_success));
        }
        if (debugInfo.get("open-url") != null) {
            String time = debugInfo.get("open-url");
            long openPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(openPts)) + getString(R.string.log_open_url_success));
        }
        if (debugInfo.get("find-stream") != null) {
            String time = debugInfo.get("find-stream");
            VcPlayerLog.d(TAG + "lfj0914", "find-Stream time =" + time + " , createpts = " + createPts);
            long findPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(findPts)) + getString(R.string.log_request_stream_success));
        }
        if (debugInfo.get("open-stream") != null) {
            String time = debugInfo.get("open-stream");
            VcPlayerLog.d(TAG + "lfj0914", "open-Stream time =" + time + " , createpts = " + createPts);
            long openPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(openPts)) + getString(R.string.log_start_open_stream));
        }
        logStrs.add(format.format(new Date()) + getString(R.string.log_first_frame_played));
    }

    private static class MyErrorListener implements IAliyunVodPlayer.OnErrorListener {

        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MyErrorListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }

        @Override
        public void onError(int arg0, int arg1, String msg) {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onError(arg0, arg1, msg);
            }
        }
    }

    private void onError(int arg0, int arg1, String msg) {

        if (aliyunVodPlayer != null) {
            aliyunVodPlayer.stop();
        }

        if (arg0 == AliyunErrorCode.ALIVC_ERR_INVALID_INPUTFILE.getCode()) {
            //当播放本地报错4003的时候，可能是文件地址不对，也有可能是没有权限。
            //如果是没有权限导致的，就做一个权限的错误提示。其他还是正常提示：
            int storagePermissionRet = ContextCompat.checkSelfPermission(DemoChangeUrlActivity.this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
            if (storagePermissionRet != PackageManager.PERMISSION_GRANTED) {
                Toast.makeText(DemoChangeUrlActivity.this.getApplicationContext(), getString(R.string.toast_fail_msg) + getString(R.string.toast_no_local_storage_permission), Toast.LENGTH_SHORT).show();
                return;
            }
        }

        Toast.makeText(DemoChangeUrlActivity.this.getApplicationContext(), getString(R.string.toast_fail_msg) + msg, Toast.LENGTH_SHORT).show();
    }

    private static class MyCompletionListener implements IAliyunVodPlayer.OnCompletionListener {
        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MyCompletionListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }

        @Override
        public void onCompletion() {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onCompletion();
            }
        }
    }

    private void onCompletion() {
        Log.d(TAG, "onCompletion--- ");
        isCompleted = true;
        inSeek = false;
        showVideoProgressInfo();
        stopUpdateTimer();

        //播放结束后切换数据源
        if (mLocalSource == videoSource1) {
            videoBtn2.setChecked(true);
        } else if (mLocalSource == videoSource2) {
            videoBtn3.setChecked(true);
        } else if (mLocalSource == videoSource3) {
            videoBtn4.setChecked(true);
        } else if (mLocalSource == videoSource4) {
            videoBtn5.setChecked(true);
        } else if (mLocalSource == videoSource5) {
            videoBtn1.setChecked(true);
        }

    }

    private static class MySeekCompleteListener implements IAliyunVodPlayer.OnSeekCompleteListener {

        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MySeekCompleteListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }

        @Override
        public void onSeekComplete() {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onSeekComplete();
            }
        }
    }

    private void onSeekComplete() {
        logStrs.add(format.format(new Date()) + getString(R.string.log_seek_completed));
        inSeek = false;
    }

    private static class MyStoppedListener implements IAliyunVodPlayer.OnStoppedListener {
        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MyStoppedListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }


        @Override
        public void onStopped() {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onStopped();
            }
        }
    }

    private void onStopped() {
        logStrs.add(format.format(new Date()) + getString(R.string.log_play_stopped));
        if (replay) {
            logStrs.add(format.format(new Date()) + getString(R.string.log_start_get_data));
            if (mLocalSource != null) {
                aliyunVodPlayer.prepareAsync(mLocalSource);
            }
        }
        replay = false;
        inSeek = false;
    }

    private static class MyChangeQualityListener implements IAliyunVodPlayer.OnChangeQualityListener {

        private WeakReference<DemoChangeUrlActivity> activityWeakReference;

        public MyChangeQualityListener(DemoChangeUrlActivity skinActivity) {
            activityWeakReference = new WeakReference<DemoChangeUrlActivity>(skinActivity);
        }

        @Override
        public void onChangeQualitySuccess(String finalQuality) {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onChangeQualitySuccess(finalQuality);
            }
        }

        @Override
        public void onChangeQualityFail(int code, String msg) {
            DemoChangeUrlActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onChangeQualityFail(code, msg);
            }
        }

    }

    private void onChangeQualitySuccess(String finalQuality) {
        Log.d(TAG, "onChangeQualitySuccess");
        inSeek = false;
        logStrs.add(format.format(new Date()) + getString(R.string.log_change_quality_success));
        showVideoProgressInfo();
    }


    private void onChangeQualityFail(int code, String msg) {
        Log.d(TAG, "onChangeQualityFail。。。" + code + " ,  " + msg);
        logStrs.add(format.format(new Date()) + getString(R.string.log_change_quality_fail));
    }


    private void setPlaySource() {
        mLocalSource = videoSource1;
    }

    private void showVideoProgressInfo() {
        if ((aliyunVodPlayer.getPlayerState().equals(AliyunVodPlayer.PlayerState.Started)
                || aliyunVodPlayer.getPlayerState().equals(AliyunVodPlayer.PlayerState.Replay) ||
                aliyunVodPlayer.getPlayerState().equals(AliyunVodPlayer.PlayerState.Completed))
                && !inSeek) {
            int curPosition = (int) aliyunVodPlayer.getCurrentPosition();
            positionTxt.setText(Formatter.formatTime(curPosition));
            int duration = (int) aliyunVodPlayer.getDuration();
            durationTxt.setText(Formatter.formatTime(duration));
            Log.d(TAG, "lfj0918 duration = " + duration + " , curPosition = " + curPosition);
            int bufferPosition = aliyunVodPlayer.getBufferingPosition();
            progressBar.setMax(duration);
            progressBar.setSecondaryProgress(bufferPosition);
            progressBar.setProgress(curPosition);
        }

        startUpdateTimer();
    }

    private void startUpdateTimer() {
        Log.d(TAG, "setUpdaterListener--- ");
        progressUpdateTimer.removeMessages(0);
        progressUpdateTimer.sendEmptyMessageDelayed(0, 1000);
    }

    private void stopUpdateTimer() {
        progressUpdateTimer.removeMessages(0);
    }

    private Handler progressUpdateTimer = new Handler() {
        @Override
        public void handleMessage(Message msg) {

            showVideoProgressInfo();
        }
    };

    @Override
    protected void onResume() {
        super.onResume();
        //保存播放器的状态，供resume恢复使用。
        resumePlayerState();
    }

    @Override
    protected void onStop() {
        super.onStop();
        //onStop中记录下来的状态，在这里恢复使用
        savePlayerState();
    }


    //用来记录前后台切换时的状态，以供恢复。
    private IAliyunVodPlayer.PlayerState mPlayerState;

    private void resumePlayerState() {
        if (mPlayerState == IAliyunVodPlayer.PlayerState.Paused) {
            aliyunVodPlayer.pause();
        } else if (mPlayerState == IAliyunVodPlayer.PlayerState.Started) {
            aliyunVodPlayer.start();
            pauseBtn.setText(R.string.pause_button);
        }
    }

    private void savePlayerState() {
        mPlayerState = aliyunVodPlayer.getPlayerState();
        if (aliyunVodPlayer.isPlaying()) {
            //然后再暂停播放器
            aliyunVodPlayer.pause();
        }
    }

    @Override
    protected void onDestroy() {
        aliyunVodPlayer.stop();
        aliyunVodPlayer.release();
        stopUpdateTimer();
        progressUpdateTimer = null;

        super.onDestroy();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        int orientation = getResources().getConfiguration().orientation;
        if (orientation == Configuration.ORIENTATION_PORTRAIT) {                //转为竖屏了。
            //显示状态栏
            this.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
            surfaceView.setSystemUiVisibility(View.SYSTEM_UI_FLAG_VISIBLE);

            //设置view的布局，宽高之类
            ViewGroup.LayoutParams surfaceViewLayoutParams = surfaceView.getLayoutParams();
            surfaceViewLayoutParams.height = (int) (ScreenUtils.getWidth(this) * 9.0f / 16);
            surfaceViewLayoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT;

        } else if (orientation == Configuration.ORIENTATION_LANDSCAPE) {                //转到横屏了。
            //隐藏状态栏
            this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
            surfaceView.setSystemUiVisibility(View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_FULLSCREEN);
            //设置view的布局，宽高
            ViewGroup.LayoutParams surfaceViewLayoutParams = surfaceView.getLayoutParams();
            surfaceViewLayoutParams.height = ViewGroup.LayoutParams.MATCH_PARENT;
            surfaceViewLayoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT;

        }
    }
}

