package com.aliyun.vodplayer.demo.activity.advance;

import android.animation.ValueAnimator;
import android.graphics.SurfaceTexture;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Surface;
import android.view.TextureView;
import android.view.View;
import android.view.ViewTreeObserver;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.alivc.player.AliVcMediaPlayer;
import com.alivc.player.AliyunErrorCode;
import com.alivc.player.MediaPlayer;
import com.alivc.player.VcPlayerLog;
import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.Contants;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.LiveAnswerView;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.SuccessDialog;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.TipDialog;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.AnswerInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.AnswerRequestInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.QuestionInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.UserCountInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.AnswerRequest;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.GetQuestionReportRequest;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.GetQuestionRequest;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.GetUserCountRequest;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.demo.utils.LongSoundPlayer;
import com.aliyun.vodplayer.demo.utils.OnClickListener;
import com.aliyun.vodplayer.demo.utils.SoundPlayer;
import com.aliyun.vodplayer.utils.BaseRequest;
import com.aliyun.vodplayer.utils.JsonUtil;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.ref.WeakReference;

public class DemoLiveAnswerActivity extends BaseAppCompatActivity {

    private static final String TAG = DemoLiveAnswerActivity.class.getSimpleName();
    private LiveAnswerView mLiveAnswerView;
    private ImageView mBackView;
    private TextView mUserNumsView;

    private TextureView surfaceView;

    private AliVcMediaPlayer aliVcMediaPlayer;
    private Surface videoSurface;

    private ValueAnimator maxSurfaceAnim;
    private ValueAnimator minSurfaceAnim;

    private int maxSurfaceWith = -1;
    private int maxSurfaceHeight = -1;
    private int minSurfaceWith = -1;
    private int minSurfaceHeight = -1;
    private int minSurfaceMagin = -1;


    private String liveUrl = null;
    private String liveId = null;
    private String userId = null;
    private boolean survivor = true;

    private boolean canAnswer = true;

    private LongSoundPlayer questionSoundPlayer = null;
    private SoundPlayer loseSoundPlayer = null;
    private SoundPlayer rightSoundPlayer = null;
    private SoundPlayer tapSoundPlayer = null;
    private SoundPlayer wrongSoundPlayer = null;


    private MyHandler userCountTimer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_live_answer);

        findAllViews();
        getIntentData();
        initSurfaceAnim();
        setEventListener();
        initSoundPlayer();
        initLivePlayer();
        requestUserCountTimely();
        showNotServivor();
    }


    private void showNotServivor() {
        if (!survivor) {
            final TipDialog dialog = new TipDialog(DemoLiveAnswerActivity.this);
            dialog.setOnBtnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialog.dismiss();
                }
            });
            dialog.showMode(getString(R.string.live_answer_has_started), getString(R.string.live_answer_i_know));
        }
    }

    private void initSoundPlayer() {
        questionSoundPlayer = new LongSoundPlayer(this, R.raw.question);
        loseSoundPlayer = new SoundPlayer(this, R.raw.results);
        rightSoundPlayer = new SoundPlayer(this, R.raw.right);
        tapSoundPlayer = new SoundPlayer(this, R.raw.tap);
        wrongSoundPlayer = new SoundPlayer(this, R.raw.wrong);
    }

    private void releaseSoundPlayer() {
        questionSoundPlayer.release();
        loseSoundPlayer.release();
        rightSoundPlayer.release();
        tapSoundPlayer.release();
        wrongSoundPlayer.release();
    }


    private static class MyHandler extends Handler {

        private WeakReference<DemoLiveAnswerActivity> activityWeakReference;

        public MyHandler(DemoLiveAnswerActivity answerActivity) {
            activityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }

        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);

            DemoLiveAnswerActivity answerActivity = activityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.handleMessage(msg);
            }
        }
    }

    private void handleMessage(Message msg) {
        requestUserCountTimely();
    }

    private void requestUserCountTimely() {
        GetUserCountRequest.UserCountParam param = new GetUserCountRequest.UserCountParam();
        param.liveId = liveId;
        param.userId = userId;

        final GetUserCountRequest request = new GetUserCountRequest(this, param, new UserCountRequestListener(this));
        request.get();

        if (userCountTimer == null) {
            userCountTimer = new MyHandler(DemoLiveAnswerActivity.this);
        }

        userCountTimer.sendEmptyMessageDelayed(0, 5 * 1000);//5秒刷新一次
    }

    private static class UserCountRequestListener implements BaseRequest.OnRequestListener<UserCountInfo> {

        private WeakReference<DemoLiveAnswerActivity> activityWeakReference;

        public UserCountRequestListener(DemoLiveAnswerActivity activity) {
            activityWeakReference = new WeakReference<DemoLiveAnswerActivity>(activity);
        }

        @Override
        public void onSuccess(UserCountInfo requestInfo, String requestId) {
            DemoLiveAnswerActivity answerActivity = activityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.updateUserCount(requestInfo);
            }
        }

        @Override
        public void onFail(int code, String msg, String requestId) {
            DemoLiveAnswerActivity answerActivity = activityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.updateUserCount(null);
            }
        }
    }

    private void updateUserCount(UserCountInfo requestInfo) {
        int userCount = 0;
        if (requestInfo != null) {
            userCount = requestInfo.userCount;
        }

        mUserNumsView.setText(userCount + "人");
    }

    private void getIntentData() {
        userId = Contants.USER_ID;
        liveUrl = Contants.LIVE_URL;
        liveId = Contants.getLiveId();
        survivor = Contants.SERVIOR;
        canAnswer = survivor;
        mLiveAnswerView.setServivor(survivor);
    }

    private void findAllViews() {
        surfaceView = (TextureView) findViewById(R.id.surfaceView);
        mLiveAnswerView = (LiveAnswerView) findViewById(R.id.live_answer_view);
        mBackView = (ImageView) findViewById(R.id.back);
        mUserNumsView = (TextView) findViewById(R.id.user_nums);
    }

    private void setEventListener() {
        mBackView.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }
                //TODO 是否要友好的提示一下？
                finish();
            }
        });

        mLiveAnswerView.setOnTimeRemainListener(new LiveAnswerView.OnTimeRemainListener() {
            @Override
            public void decountEnd() {
                //不答题的情况下
                if (mLiveAnswerView.getMyAnswer() == null) {
                    canAnswer = false;
                }
                showQuestion = false;
                mLiveAnswerView.hide();
                maxLiveSurface();
            }

            @Override
            public void resultEnd() {
                if (!showQuestion) {
                    mLiveAnswerView.hide();
                    maxLiveSurface();
                }
            }

            @Override
            public void onChoose(AnswerRequestInfo answerRequestInfo) {
                sendMyAnswerToServer(answerRequestInfo);
            }
        });

        surfaceView.getViewTreeObserver().addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {
            @Override
            public void onGlobalLayout() {
                if (maxSurfaceWith < 0) {

                    maxSurfaceWith = surfaceView.getWidth();
                    maxSurfaceHeight = surfaceView.getHeight();

                    minSurfaceWith = maxSurfaceWith / 4;
                    minSurfaceHeight = maxSurfaceHeight / 4;
                    minSurfaceMagin = maxSurfaceWith / 15;
                }
            }
        });
        surfaceView.setSurfaceTextureListener(new TextureView.SurfaceTextureListener() {
            @Override
            public void onSurfaceTextureAvailable(SurfaceTexture surface, int width, int height) {

                videoSurface = new Surface(surface);
                if (aliVcMediaPlayer == null) {
                    initLivePlayer();
                } else {
                    aliVcMediaPlayer.setVideoSurface(videoSurface);
                }
            }

            @Override
            public void onSurfaceTextureSizeChanged(SurfaceTexture surface, int width, int height) {
                aliVcMediaPlayer.setSurfaceChanged();
            }

            @Override
            public boolean onSurfaceTextureDestroyed(SurfaceTexture surface) {
                return true;
            }

            @Override
            public void onSurfaceTextureUpdated(SurfaceTexture surface) {

            }
        });

    }

    private void initLivePlayer() {

        aliVcMediaPlayer = new AliVcMediaPlayer(DemoLiveAnswerActivity.this, videoSurface);
        aliVcMediaPlayer.enableNativeLog();
        aliVcMediaPlayer.setSEIDataListener(new MySeiDataListener(DemoLiveAnswerActivity.this));
        aliVcMediaPlayer.setFrameInfoListener(new MyFrameInfoListener(DemoLiveAnswerActivity.this));
        aliVcMediaPlayer.setErrorListener(new MyErrorListener(DemoLiveAnswerActivity.this));
        aliVcMediaPlayer.setCompletedListener(new MyCompleteListener(DemoLiveAnswerActivity.this));
        //保证全屏显示。
        aliVcMediaPlayer.setVideoScalingMode(MediaPlayer.VideoScalingMode.VIDEO_SCALING_MODE_SCALE_TO_FIT_WITH_CROPPING);
        aliVcMediaPlayer.setMaxBufferDuration(3 * 1000);
        aliVcMediaPlayer.prepareAndPlay(liveUrl);

    }


    private class MyCompleteListener implements MediaPlayer.MediaPlayerCompletedListener {


        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        private MyCompleteListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }

        @Override
        public void onCompleted() {
            VcPlayerLog.d("DemoLiveAnserActivity", "播放结束： ");
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onCompleted();
            }
        }
    }

    private void onCompleted() {
        Toast.makeText(getApplicationContext(), R.string.live_answer_finished, Toast.LENGTH_SHORT).show();
        finish();
    }

    private void releasePlayer() {
        if (aliVcMediaPlayer != null) {
            aliVcMediaPlayer.stop();
            aliVcMediaPlayer.destroy();
        }
    }


    private static class MyAnimatorUpdateListener implements ValueAnimator.AnimatorUpdateListener {
        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        public MyAnimatorUpdateListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }


        @Override
        public void onAnimationUpdate(ValueAnimator animation) {
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onAnimationUpdate(animation);
            }
        }
    }

    private void onAnimationUpdate(ValueAnimator animation) {
        if (animation == maxSurfaceAnim) {
            float value = (float) animation.getAnimatedValue();
            RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) surfaceView.getLayoutParams();

            layoutParams.width = (int) ((maxSurfaceWith - minSurfaceWith) * value + minSurfaceWith);
            layoutParams.height = (int) ((maxSurfaceHeight - minSurfaceHeight) * value + minSurfaceHeight);

            layoutParams.bottomMargin = (int) ((0 - minSurfaceMagin) * value + minSurfaceMagin);
            layoutParams.rightMargin = (int) ((0 - minSurfaceMagin) * value + minSurfaceMagin);

            surfaceView.setLayoutParams(layoutParams);
        } else if (animation == minSurfaceAnim) {
            float value = (float) animation.getAnimatedValue();
            RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) surfaceView.getLayoutParams();

            layoutParams.width = (int) ((minSurfaceWith - maxSurfaceWith) * value + maxSurfaceWith);
            layoutParams.height = (int) ((minSurfaceHeight - maxSurfaceHeight) * value + maxSurfaceHeight);

            layoutParams.bottomMargin = (int) ((minSurfaceMagin - 0) * value + 0);
            layoutParams.rightMargin = (int) ((minSurfaceMagin - 0) * value + 0);

            surfaceView.setLayoutParams(layoutParams);
        }
    }


    private void initSurfaceAnim() {

        maxSurfaceAnim = ValueAnimator.ofFloat(0, 1);
        maxSurfaceAnim.setDuration(400);
        maxSurfaceAnim.addUpdateListener(new MyAnimatorUpdateListener(this));

        minSurfaceAnim = ValueAnimator.ofFloat(0, 1);
        minSurfaceAnim.setDuration(400);
        minSurfaceAnim.addUpdateListener(new MyAnimatorUpdateListener(this));
    }

    private static class MyErrorListener implements MediaPlayer.MediaPlayerErrorListener {

        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        private MyErrorListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }


        @Override
        public void onError(int i, String s) {
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onError(i, s);
            }
        }
    }

    private int retryNum = 0;

    private void onError(int code, String msg) {
        VcPlayerLog.d("DemoLiveAnserActivity", "播放出错：code = " + code);
        if (code == AliyunErrorCode.ALIVC_ERROR_LOADING_TIMEOUT.getCode()) {
            if (retryNum > 5) {
                Toast.makeText(getApplicationContext(), R.string.live_answer_play_fail, Toast.LENGTH_SHORT).show();
            } else {

                Toast.makeText(getApplicationContext(), R.string.live_answer_retry, Toast.LENGTH_SHORT).show();
                //尝试重试
                retryNum++;
                if (aliVcMediaPlayer != null) {
                    aliVcMediaPlayer.stop();
                    aliVcMediaPlayer.prepareAndPlay(liveUrl);
                }

            }
        }
    }

    private static class MyFrameInfoListener implements MediaPlayer.MediaPlayerFrameInfoListener {
        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        private MyFrameInfoListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }


        @Override
        public void onFrameInfoListener() {
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onFrameInfoListener();
            }
        }
    }

    private void onFrameInfoListener() {
        retryNum = 0;//重置
    }

    private static class MySeiDataListener implements MediaPlayer.MediaPlayerSEIDataListener {

        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        private MySeiDataListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }

        @Override
        public void onSeiUserUnregisteredData(String data) {
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onSEI_userUnregisteredData(data);
            }
        }
    }

    private void onSEI_userUnregisteredData(String data) {
        VcPlayerLog.d("DemoLiveAnswerActivity", "onSEI_userUnregisteredData :SEI data = " + data);
//        Toast.makeText(getApplicationContext(), "SEI ：" + data, Toast.LENGTH_SHORT).showMode();
        praseData(data);
    }

    public static class SeiInfo {
        public String questionId;
        public String type;
        public int showTime;

        public static SeiInfo getFromJson(JSONObject jsonObject) {
            if (jsonObject == null) {
                return null;

            }

            SeiInfo seiInfo = new SeiInfo();
            seiInfo.questionId = JsonUtil.getString(jsonObject, "questionId");
            seiInfo.type = JsonUtil.getString(jsonObject, "type");
            seiInfo.showTime = JsonUtil.getInt(jsonObject, "showTime");

            return seiInfo;
        }
    }

    private void praseData(String data) {
        JSONObject jsonObject = null;
        try {
            jsonObject = new JSONObject(data);
        } catch (JSONException e) {
            VcPlayerLog.e(TAG, "e : " + e.getMessage());
        }

        SeiInfo seiInfo = SeiInfo.getFromJson(jsonObject);
        if (seiInfo == null) {
            Toast.makeText(getApplicationContext(), R.string.live_answer_sei_data_wrong, Toast.LENGTH_SHORT).show();
            return;
        }

        if (seiInfo.type.equals("startAnswer")) {
            requestQuestion(seiInfo.questionId);
        } else if (seiInfo.type.equals("showResult")) {
            if (seiInfo.questionId.equals("012") && canAnswer) {
                //表示第12题都答对了。
                SuccessDialog dialog = new SuccessDialog(DemoLiveAnswerActivity.this);
                dialog.show();
            } else {
                requestQuestionReport(seiInfo.questionId, seiInfo.showTime);
            }
        }

    }

    private void sendMyAnswerToServer(AnswerRequestInfo answerRequestInfo) {
        //发送我的答题结果给服务器。
        if (answerRequestInfo == null) {
            canAnswer = false;
            return;
        }

        tapSoundPlayer.play();

        AnswerRequest.AnswerParam answerParam = new AnswerRequest.AnswerParam();
        answerParam.answerId = answerRequestInfo.answerId;
        answerParam.liveId = liveId;
        answerParam.questionId = answerRequestInfo.questionId;
        answerParam.userId = userId;

        AnswerRequest answerRequest = new AnswerRequest(getApplicationContext(),
                answerParam, new BaseRequest.OnRequestListener<AnswerInfo>() {
            @Override
            public void onSuccess(AnswerInfo requestInfo, String requestId) {
                canAnswer = requestInfo.isCorrect;
            }

            @Override
            public void onFail(int code, String msg, String requestId) {
                Toast.makeText(getApplicationContext(), getString(R.string.live_answer_answer_fail) + msg, Toast.LENGTH_SHORT).show();
                //判断结果,正确可以继续答题，不正确不能答题
                canAnswer = false;
            }
        });
        answerRequest.get();
    }


    private void requestQuestion(String questionId) {

        GetQuestionRequest.GetQuestionParam getQuestionParam = new GetQuestionRequest.GetQuestionParam();
        getQuestionParam.liveId = liveId;
        getQuestionParam.questionId = questionId;

        GetQuestionRequest getQuestionRequest = new GetQuestionRequest(getApplicationContext(),
                getQuestionParam, new BaseRequest.OnRequestListener<QuestionInfo>() {
            @Override
            public void onSuccess(QuestionInfo questionInfo, String requestId) {

                if (questionInfo != null) {
                    if (questionInfo.remainSeconds > 10) {
                        questionInfo.remainSeconds = 10;
                    }

                    if (questionInfo.remainSeconds <= 0) {
                        questionInfo.remainSeconds = 5;
                    }

                    mLiveAnswerView.setQuestion(questionInfo, canAnswer);
                    mLiveAnswerView.show();
                    showQuestion = true;
                    miniLiveSurface();

                    questionSoundPlayer.play();
                } else {
                    canAnswer = false;
                    Toast.makeText(getApplicationContext(), R.string.live_answer_get_question_fail, Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onFail(int code, String msg, String requestId) {
                Toast.makeText(getApplicationContext(), getString(R.string.live_answer_get_question_fail) + msg, Toast.LENGTH_SHORT).show();
            }
        });
        getQuestionRequest.get();
    }

    private boolean showQuestion = false;

    private void requestQuestionReport(String questionId, final int showTime) {

        GetQuestionReportRequest.GetQuestionReportParam getQuestionParam = new GetQuestionReportRequest.GetQuestionReportParam();
        getQuestionParam.liveId = liveId;
        getQuestionParam.questionId = questionId;

        GetQuestionReportRequest reportRequest = new GetQuestionReportRequest(getApplicationContext(),
                getQuestionParam, new BaseRequest.OnRequestListener<QuestionInfo>() {
            @Override
            public void onSuccess(QuestionInfo questionReportInfo, String requestId) {
                int finalShowTime = showTime;
                if (finalShowTime <= 0) {
                    finalShowTime = 5;
                }

                LiveAnswerView.ResultStatus resultStatus = mLiveAnswerView.setResult(questionReportInfo, canAnswer, finalShowTime);
                mLiveAnswerView.show();
                showQuestion = false;
                miniLiveSurface();

                if (LiveAnswerView.ResultStatus.Right == resultStatus) {
                    rightSoundPlayer.play();
                } else if (LiveAnswerView.ResultStatus.Wrong == resultStatus) {
                    wrongSoundPlayer.play();
                } else if (LiveAnswerView.ResultStatus.Lose == resultStatus) {
                    loseSoundPlayer.play();
                } else {

                }

            }

            @Override
            public void onFail(int code, String msg, String requestId) {
                Toast.makeText(getApplicationContext(), getString(R.string.live_answer_getresult_fail) + msg, Toast.LENGTH_SHORT).show();
            }
        });
        reportRequest.get();

    }


    private void miniLiveSurface() {
        minSurfaceAnim.start();
    }

    private void maxLiveSurface() {
        maxSurfaceAnim.start();
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        releaseSoundPlayer();
        releasePlayer();
        userCountTimer.removeMessages(0);
    }


}
