package com.aliyun.vodplayer.demo.activity.advance;

import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.IdRes;
import android.support.v7.app.AlertDialog;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.demo.utils.Formatter;
import com.aliyun.vodplayer.demo.utils.LiveSeekBar;
import com.aliyun.vodplayer.demo.utils.OnClickListener;
import com.aliyun.vodplayer.media.AliyunLiveTimeShift;
import com.aliyun.vodplayer.media.AliyunVodPlayer;
import com.aliyun.vodplayer.media.IAliyunVodPlayer;
import com.aliyun.vodplayerview.utils.NetWatchdog;

import java.lang.ref.WeakReference;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Live demo
 */
public class DemoTimeShiftLiveActivity extends BaseAppCompatActivity {

    private static final String TAG = DemoTimeShiftLiveActivity.class.getSimpleName();

    private SimpleDateFormat format = new SimpleDateFormat("HH:mm:ss.SS");

    private SurfaceView mSurfaceView;


    private TextView liveTimeTxt;
    private TextView playTimeTxt;
    private LiveSeekBar liveSeekBar;
    private TextView endTimeTxt;

    private Button playBtn;
    private Button pauseBtn;
    private Button stopBtn;


    private CheckBox muteOnBtn;

    private RadioGroup scaleModeGroup;
    private RadioButton scaleModeFit;
    private RadioButton scaleModeFill;

    private SeekBar brightnessBar;
    private SeekBar volumeBar;

    private TextView videoWidthTxt;
    private TextView videoHeightTxt;


    private boolean mMute = false;
    private List<String> logStrs = new ArrayList<>();

    private AliyunVodPlayer mPlayer;

    private boolean isCompleted = false;

    //    private String mUrl;
    private AliyunLiveTimeShift mAliyunLiveTimeShift;

    private NetWatchdog netWatchdog;

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_log, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.log) {

            LayoutInflater inflater = LayoutInflater.from(this);
            View view = inflater.inflate(R.layout.view_log, null);

            TextView textview = (TextView) view.findViewById(R.id.log);
            if (mPlayer != null) {
                for (String log : logStrs) {
                    textview.append("     " + log + "\n");
                }
            }
            AlertDialog.Builder alertDialog = new AlertDialog.Builder(this);
            alertDialog.setTitle(getString(R.string.player_log));
            alertDialog.setView(view);
            alertDialog.setPositiveButton("OK", null);
            AlertDialog alert = alertDialog.create();
            alert.show();
        }
        return true;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        setContentView(R.layout.activity_demo_time_shift);

        mSurfaceView = (SurfaceView) findViewById(R.id.surfaceView);


        liveTimeTxt = (TextView) findViewById(R.id.current_live_time);
        playTimeTxt = (TextView) findViewById(R.id.current_play_time);
        liveSeekBar = (LiveSeekBar) findViewById(R.id.seekbar);
        endTimeTxt = (TextView) findViewById(R.id.endTime);


        playBtn = (Button) findViewById(R.id.play);
        stopBtn = (Button) findViewById(R.id.stop);
        pauseBtn = (Button) findViewById(R.id.pause);
        findViewById(R.id.progress_layout).setVisibility(View.GONE);

        //不可见的view们
        findViewById(R.id.replay).setVisibility(View.GONE);
        findViewById(R.id.snapshot).setVisibility(View.GONE);
        findViewById(R.id.auto_play_layout).setVisibility(View.GONE);
        findViewById(R.id.change_quality_layout).setVisibility(View.GONE);
        findViewById(R.id.speed_layout).setVisibility(View.GONE);
        findViewById(R.id.rotate_layout).setVisibility(View.GONE);
        findViewById(R.id.mirror_layout).setVisibility(View.GONE);


        muteOnBtn = (CheckBox) findViewById(R.id.muteOn);

        muteOnBtn.setOnCheckedChangeListener(new CheckBox.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (buttonView.isChecked()) {
                    mMute = true;
                    if (mPlayer != null) {
                        mPlayer.setMuteMode(mMute);
                    }
                    volumeBar.setProgress(0);
                } else {
                    mMute = false;
                    if (mPlayer != null) {
                        mPlayer.setMuteMode(mMute);
                    }
                    volumeBar.setProgress(mPlayer.getVolume());
                }
            }

        });

        scaleModeGroup = (RadioGroup) findViewById(R.id.scalingMode);
        scaleModeFit = (RadioButton) findViewById(R.id.fit);
        scaleModeFill = (RadioButton) findViewById(R.id.fill);

        scaleModeFit.setChecked(true);

        scaleModeGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, @IdRes int checkedId) {
                if (checkedId == scaleModeFit.getId()) {
                    if (mPlayer != null) {
                        mPlayer.setVideoScalingMode(IAliyunVodPlayer.VideoScalingMode.VIDEO_SCALING_MODE_SCALE_TO_FIT);
                    }
                } else if (checkedId == scaleModeFill.getId()) {
                    if (mPlayer != null) {
                        mPlayer.setVideoScalingMode(IAliyunVodPlayer.VideoScalingMode.VIDEO_SCALING_MODE_SCALE_TO_FIT_WITH_CROPPING);
                    }
                }
            }
        });

        brightnessBar = (SeekBar) findViewById(R.id.brightnessProgress);
        volumeBar = (SeekBar) findViewById(R.id.volumeProgress);

        videoWidthTxt = (TextView) findViewById(R.id.width);
        videoHeightTxt = (TextView) findViewById(R.id.height);

        playBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }
                logStrs.add(format.format(new Date()) + getString(R.string.log_start_get_data));

                replay();

                if (mMute) {
                    mPlayer.setMuteMode(mMute);
                }
                brightnessBar.setProgress(mPlayer.getScreenBrightness());
                logStrs.add(format.format(new Date()) + getString(R.string.log_strart_play));
                volumeBar.setProgress(mPlayer.getVolume());
            }
        });
        stopBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }
                stop();
            }
        });
        pauseBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isDoubleClick()) {
                    return;
                }
                IAliyunVodPlayer.PlayerState playerState = mPlayer.getPlayerState();
                if (playerState == IAliyunVodPlayer.PlayerState.Started) {
                    pause();
                    pauseBtn.setText(R.string.resume_button);
                } else if (playerState == IAliyunVodPlayer.PlayerState.Paused) {
                    resume();
                    pauseBtn.setText(R.string.pause_button);
                }
            }
        });

        liveSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    if (isCompleted) {
                        inSeek = false;
                    } else {
                        inSeek = true;
                    }

                    long seekTime = liveSeekBar.getProgress() + mShiftStartTime;
                    playTimeTxt.setText(Formatter.formatDate(seekTime));
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
                if (mPlayer != null) {

                    logStrs.add(format.format(new Date()) + getString(R.string.log_seek_start));

                    long seekTime = liveSeekBar.getProgress() + mShiftStartTime;
                    mPlayer.seekToLiveTime(seekTime);

                    if (isCompleted) {
                        inSeek = false;
                    } else {
                        inSeek = true;
                    }

                    Log.d("lfj0929", "onStopTrackingTouch , inSeek= " + inSeek);
                }
            }
        });

        brightnessBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {

                if (fromUser && mPlayer != null) {
                    mPlayer.setScreenBrightness(progress);
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        volumeBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {

                if (fromUser && mPlayer != null) {
                    mPlayer.setVolume(progress);
                    muteOnBtn.setChecked(false);
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });


        mSurfaceView.getHolder().addCallback(new SurfaceHolder.Callback() {

            public void surfaceCreated(SurfaceHolder holder) {
                holder.setType(SurfaceHolder.SURFACE_TYPE_GPU);
                holder.setKeepScreenOn(true);
                Log.d(TAG, "AlivcPlayer onSurfaceCreated." + mPlayer);

                // Important: surfaceView changed from background to front, we need reset surface to mediaplayer.
                // 对于从后台切换到前台,需要重设surface;部分手机锁屏也会做前后台切换的处理
                if (mPlayer != null) {
                    mPlayer.setSurface(mSurfaceView.getHolder().getSurface());
                }

                Log.d(TAG, "AlivcPlayeron SurfaceCreated over.");
            }

            public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
                Log.d(TAG, "onSurfaceChanged is valid ? " + holder.getSurface().isValid());
                if (mPlayer != null) {
                    mPlayer.surfaceChanged();
                }
            }

            public void surfaceDestroyed(SurfaceHolder holder) {
                Log.d(TAG, "onSurfaceDestroy.");
            }
        });

        initVodPlayer();
        setPlaySource();
        netWatchdog = new NetWatchdog(this);
        netWatchdog.setNetChangeListener(new MyNetChangeListener(this));
        netWatchdog.startWatch();
    }

    private static class MyNetChangeListener implements NetWatchdog.NetChangeListener {

        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MyNetChangeListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }

        @Override
        public void onWifiTo4G() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onWifiTo4G();
            }
        }

        @Override
        public void on4GToWifi() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.on4GToWifi();
            }
        }

        @Override
        public void onNetDisconnected() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onNetDisconnected();
            }
        }
    }


    private void onWifiTo4G() {
        if (mPlayer.isPlaying()) {
            pause();
        }
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(DemoTimeShiftLiveActivity.this);
        alertDialog.setTitle(getString(R.string.net_change_to_4g));
        alertDialog.setMessage(getString(R.string.net_change_to_continue));
        alertDialog.setPositiveButton(getString(R.string.ok), new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                replay();
            }
        });
        alertDialog.setNegativeButton(getString(R.string.no), null);
        AlertDialog alert = alertDialog.create();
        alert.show();

        Toast.makeText(DemoTimeShiftLiveActivity.this.getApplicationContext(), R.string.net_change_to_4g, Toast.LENGTH_SHORT).show();
    }

    private void on4GToWifi() {
        Toast.makeText(DemoTimeShiftLiveActivity.this.getApplicationContext(), R.string.net_change_to_wifi, Toast.LENGTH_SHORT).show();

    }

    private void onNetDisconnected() {
        Toast.makeText(DemoTimeShiftLiveActivity.this.getApplicationContext(), R.string.net_disconnect, Toast.LENGTH_SHORT).show();
    }


    private boolean inSeek = false;

    private void showVideoProgressInfo() {

        if (mPlayer != null) {
            mLiveTime = mPlayer.getCurrentLiveTime();
            mPlayTime = mPlayer.getCurrentTime();
            long offsetTimeLen = mShiftEndTime - mShiftStartTime;

            if (mEndTime - mLiveTime < offsetTimeLen * 0.05) {
                mEndTime = (long) (mLiveTime + offsetTimeLen * 0.1);
            }

            if (!inSeek) {
                liveTimeTxt.setText(Formatter.formatDate(mLiveTime));
                endTimeTxt.setText(Formatter.formatDate(mEndTime));
                playTimeTxt.setText(Formatter.formatDate(mPlayTime));

                liveSeekBar.setPlayProgress(mPlayTime);
                liveSeekBar.setLiveTime(mLiveTime);
                liveSeekBar.updateRange(mShiftStartTime, mEndTime);

            }

        }
        startUpdateTimer();
    }

    private void startUpdateTimer() {
        progressUpdateTimer.removeMessages(0);
        progressUpdateTimer.sendEmptyMessageDelayed(0, 1000);
    }

    private void stopUpdateTimer() {
        progressUpdateTimer.removeMessages(0);
    }

    private Handler progressUpdateTimer = new Handler() {
        @Override
        public void handleMessage(Message msg) {

            showVideoProgressInfo();
        }
    };

    private void initVodPlayer() {
        mPlayer = new AliyunVodPlayer(this);

        mPlayer.setOnPreparedListener(new MyPreparedListener(this));
        mPlayer.setOnFirstFrameStartListener(new MyFrameInfoListener(this));
        mPlayer.setOnErrorListener(new MyErrorListener(this));
        mPlayer.setOnCompletionListener(new MyPlayerCompletedListener(this));
        mPlayer.setOnSeekCompleteListener(new MySeekCompleteListener(this));
        mPlayer.setOnStoppedListner(new MyStoppedListener(this));
        mPlayer.setOnTimeShiftUpdaterListener(new MyTimeShiftUpdaterListener(this));
        mPlayer.setOnSeekLiveCompletionListener(new MySeekLiveCompletionListener(this));
        mPlayer.enableNativeLog();
    }


    private static class MySeekLiveCompletionListener implements IAliyunVodPlayer.OnSeekLiveCompletionListener {
        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MySeekLiveCompletionListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }


        @Override
        public void onSeekLiveCompletion(long playTime) {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onSeekLiveCompletion(playTime);
            }
        }
    }

    private void onSeekLiveCompletion(long playTime) {
        inSeek = false;
        mPlayTime = playTime;
    }

    private static class MyTimeShiftUpdaterListener implements IAliyunVodPlayer.OnTimeShiftUpdaterListener {
        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MyTimeShiftUpdaterListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }

        @Override
        public void onUpdater(long currentTime, long shiftStartTime, long shiftEndTime) {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onUpdater(currentTime, shiftStartTime, shiftEndTime);
            }
        }
    }

    long mLiveTime;
    long mShiftStartTime;
    long mShiftEndTime;
    long mPlayTime;
    long mEndTime = -1;

    private void onUpdater(long currentLiveTime, long shiftStartTime, long shiftEndTime) {
        mLiveTime = currentLiveTime;
        mShiftStartTime = shiftStartTime;
        mShiftEndTime = shiftEndTime;

        showVideoProgressInfo();
    }

    private static class MyErrorListener implements IAliyunVodPlayer.OnErrorListener {

        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MyErrorListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }

        @Override
        public void onError(int arg0, int arg1, String msg) {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onError(arg0, msg);
            }
        }
    }

    private void onError(int i, String msg) {
        stop();
        Toast.makeText(DemoTimeShiftLiveActivity.this.getApplicationContext(), getString(R.string.toast_fail_msg) + msg, Toast.LENGTH_SHORT).show();
    }

    private static class MyPreparedListener implements IAliyunVodPlayer.OnPreparedListener {

        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MyPreparedListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }

        @Override
        public void onPrepared() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onPrepared();
            }
        }

    }

    void onPrepared() {
        Toast.makeText(DemoTimeShiftLiveActivity.this.getApplicationContext(), R.string.toast_prepare_success, Toast.LENGTH_SHORT).show();
        logStrs.add(format.format(new Date()) + getString(R.string.log_prepare_success));

        showVideoSizeInfo();
    }

    private static class MyStoppedListener implements IAliyunVodPlayer.OnStoppedListener {

        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MyStoppedListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }


        @Override
        public void onStopped() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onStopped();
            }
        }
    }

    void onStopped() {
        logStrs.add(format.format(new Date()) + getString(R.string.log_play_stopped));
    }

    private static class MySeekCompleteListener implements IAliyunVodPlayer.OnSeekCompleteListener {


        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MySeekCompleteListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }

        @Override
        public void onSeekComplete() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onSeekCompleted();
            }
        }
    }

    void onSeekCompleted() {
        logStrs.add(format.format(new Date()) + getString(R.string.log_seek_completed));
        inSeek = false;
    }

    private static class MyPlayerCompletedListener implements IAliyunVodPlayer.OnCompletionListener {

        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MyPlayerCompletedListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }

        @Override
        public void onCompletion() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onCompleted();
            }
        }
    }

    private void onCompleted() {
        Log.d(TAG, "onCompleted--- ");
        isCompleted = true;
    }

    private static class MyFrameInfoListener implements IAliyunVodPlayer.OnFirstFrameStartListener {

        private WeakReference<DemoTimeShiftLiveActivity> activityWeakReference;

        public MyFrameInfoListener(DemoTimeShiftLiveActivity activity) {
            activityWeakReference = new WeakReference<DemoTimeShiftLiveActivity>(activity);
        }

        @Override
        public void onFirstFrameStart() {
            DemoTimeShiftLiveActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onFrameInfoListener();
            }
        }
    }


    private void onFrameInfoListener() {

        inSeek = false;

        Map<String, String> debugInfo = mPlayer.getAllDebugInfo();
        long createPts = 0;
        if (debugInfo.get("create_player") != null) {
            String time = debugInfo.get("create_player");
            createPts = (long) Double.parseDouble(time);
            logStrs.add(format.format(new Date(createPts)) + getString(R.string.log_player_create_success));
        }
        if (debugInfo.get("open-url") != null) {
            String time = debugInfo.get("open-url");
            long openPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(openPts)) + getString(R.string.log_open_url_success));
        }
        if (debugInfo.get("find-stream") != null) {
            String time = debugInfo.get("find-stream");
            Log.d(TAG + "lfj0914", "find-Stream time =" + time + " , createpts = " + createPts);
            long findPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(findPts)) + getString(R.string.log_request_stream_success));
        }
        if (debugInfo.get("open-stream") != null) {
            String time = debugInfo.get("open-stream");
            Log.d(TAG + "lfj0914", "open-Stream time =" + time + " , createpts = " + createPts);
            long openPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(openPts)) + getString(R.string.log_start_open_stream));
        }
        logStrs.add(format.format(new Date()) + getString(R.string.log_first_frame_played));
    }

    private void setPlaySource() {
        mAliyunLiveTimeShift = new AliyunLiveTimeShift();

        long currentSeconds = System.currentTimeMillis() / 1000;
        //有鉴权地址
        mAliyunLiveTimeShift.setUrl("http://pull-videocall.aliyuncs.com/timeline/test.m3u8");
        mAliyunLiveTimeShift.setTimeLineUrl("http://pull-videocall.aliyuncs.com/openapi/timeline/query?lhs_start=1&app=timeline&stream=test&format=ts&lhs_start_unix_s_0="
                + (currentSeconds - 5 * 60) + "&lhs_end_unix_s_0=" + (currentSeconds + 5 * 60));

        //无鉴权地址
//        mAliyunLiveTimeShift.setUrl("http://cctv1.cntv.cdnpe.com/timeline/cctv5td.m3u8");
//        mAliyunLiveTimeShift.setTimeLineUrl("http://cctv1.cntv.cdnpe.com/openapi/timeline/query?app=timeline&stream=cctv5td&format=ts&lhs_start_unix_s_0="
//                + ( currentSeconds -5*60 ) + "&lhs_end_unix_s_0="+ (currentSeconds + 5*60));
    }

    private void showVideoSizeInfo() {
        videoWidthTxt.setText(getString(R.string.video_width) + mPlayer.getVideoWidth() + " , ");
        videoHeightTxt.setText(getString(R.string.video_height) + mPlayer.getVideoHeight() + "   ");
    }


    private void start() {
        setPlaySource();
        if (mPlayer != null) {
            mPlayer.setAutoPlay(true);
            mPlayer.prepareAsync(mAliyunLiveTimeShift);
        }

    }

    private void pause() {
        if (mPlayer != null) {
            mPlayer.pause();
            pauseBtn.setText(R.string.resume_button);
        }
    }

    private void stop() {
        stopUpdateTimer();
        if (mPlayer != null) {
            mPlayer.stop();
        }
    }

    private void resume() {
        if (mPlayer != null) {
            mPlayer.start();
            pauseBtn.setText(R.string.pause_button);
        }
    }

    private void destroy() {
        if (mPlayer != null) {
            mPlayer.stop();
            mPlayer.release();
        }
    }

    private void replay() {
        stop();
        start();
    }

    @Override
    protected void onResume() {
        super.onResume();

    }

    @Override
    protected void onStop() {
        super.onStop();
        //when view goto background,will pausethe player, so we save the player's status here,
        // and when activity resumed, we resume the player's status.
        savePlayerState();
    }

    private void savePlayerState() {
        if (mPlayer.isPlaying()) {
            //we pause the player for not playing on the background
            // 不可见，暂停播放器
            pause();
        }
    }

    @Override
    protected void onDestroy() {
        stop();
        destroy();

        netWatchdog.stopWatch();

        super.onDestroy();
    }

}
