package com.aliyun.vodplayer.demo.activity.advance.download;

import android.content.Context;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.aliyun.vodplayer.demo.BuildConfig;
import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.activity.advance.DemoDownloadActivity;
import com.aliyun.vodplayer.demo.utils.VidStsUtil;
import com.aliyun.vodplayer.demo.utils.WrapCheckGroup;
import com.aliyun.vodplayer.downloader.AliyunDownloadInfoListener;
import com.aliyun.vodplayer.downloader.AliyunDownloadManager;
import com.aliyun.vodplayer.downloader.AliyunDownloadMediaInfo;
import com.aliyun.vodplayer.media.AliyunVidSts;
import com.aliyun.vodplayer.utils.QualityLanguage;

import java.lang.ref.WeakReference;
import java.util.List;

/**
 *
 */

public class AddDownloadView extends LinearLayout {

    private EditText vid;
    private EditText akId, akSecret, scuToken;

    private Button prepareBtn, downloadBtn;
    private WrapCheckGroup wrapCheckGroup;


    private AliyunDownloadManager downloadManager;

//    private Map<String, String> qualityList = new HashMap<>();

    public AddDownloadView(Context context) {
        super(context);
        init();
    }


    public AddDownloadView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init();
    }


    public AddDownloadView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    AliyunDownloadInfoListener downloadInfoListener;

    private void init() {
        LayoutInflater.from(getContext()).inflate(R.layout.view_add_download, this, true);
        findAllViews();
        downloadManager = AliyunDownloadManager.getInstance(getContext());
        downloadInfoListener = new MyDownloadInfoListener(this);
        downloadManager.addDownloadInfoListener(downloadInfoListener);

//        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_FLUENT, getContext().getString(R.string.alivc_fd_definition));
//        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_LOW, getContext().getString(R.string.alivc_ld_definition));
//        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_STAND, getContext().getString(R.string.alivc_sd_definition));
//        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_HIGH, getContext().getString(R.string.alivc_hd_definition));
//        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_2K, getContext().getString(R.string.alivc_k2_definition));
//        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_4K, getContext().getString(R.string.alivc_k4_definition));
//        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_ORIGINAL, getContext().getString(R.string.alivc_od_definition));
    }

    public void destory() {
        downloadManager.removeDownloadInfoListener(downloadInfoListener);
        downloadInfoListener = null;
    }


    private static class MyDownloadInfoListener implements AliyunDownloadInfoListener {

        private WeakReference<AddDownloadView> weakView;

        public MyDownloadInfoListener(AddDownloadView view) {
            weakView = new WeakReference<AddDownloadView>(view);
        }

        @Override
        public void onPrepared(List<AliyunDownloadMediaInfo> infos) {
            AddDownloadView view = weakView.get();
            if (view != null) {
                view.onPrepared(infos);
            }
        }

        @Override
        public void onStart(AliyunDownloadMediaInfo info) {

        }

        @Override
        public void onProgress(AliyunDownloadMediaInfo info, int percent) {

        }

        @Override
        public void onStop(AliyunDownloadMediaInfo info) {

        }

        @Override
        public void onCompletion(AliyunDownloadMediaInfo info) {

        }

        @Override
        public void onError(AliyunDownloadMediaInfo info, int code, String msg, String requestId) {

        }

        @Override
        public void onWait(AliyunDownloadMediaInfo outMediaInfo) {

        }

        @Override
        public void onM3u8IndexUpdate(AliyunDownloadMediaInfo outMediaInfo, int index) {

        }
    }

    void onPrepared(List<AliyunDownloadMediaInfo> infos) {
        //第二步 ； 准备结束
        showAllDownloadItems(infos);
    }

    private void showAllDownloadItems(List<AliyunDownloadMediaInfo> list) {
        if (list == null || list.isEmpty()) {
            Toast.makeText(getContext().getApplicationContext(), R.string.no_download_right, Toast.LENGTH_LONG).show();
            return;
        }

        wrapCheckGroup.removeCheckBox();
        Log.d("demo", "list size = " + list.size());

        int infoSize = list.size();
        for (int i =0 ; i < infoSize;i++) {
            AliyunDownloadMediaInfo info = list.get(i);
            String encript = info.isEncripted() == 1 ? getContext().getString(R.string.encrypted) : getContext().getString(R.string.encrypted_no);
            CheckBox item = (CheckBox) LayoutInflater.from(getContext()).inflate(R.layout.view_item_quality, null, false);
            item.setText(" " + QualityLanguage.getSaasLanguage( getContext() , info.getQuality()) + " {" + info.getFormat() + "," + formatSize(info.getSize()) + encript + "} ");
//            item.setText(" " + QualityLanguage.getMtsLanguage( getContext() , info.getQuality()) + " {" + info.getFormat() + "," + formatSize(info.getSize()) + encript + "} ");
            item.setTag(info);
            ////设置id，供自动化测试用
            int id = R.id.custom_id_min + i;
            item.setId(id);
            wrapCheckGroup.addCheckBox(item);
        }
    }

    private String formatSize(int size) {
        int kb = (int) (size / 1024f);
        if (kb < 1024) {
            return kb + "KB";
        }

        int mb = (int) (kb / 1024f);
        return mb + "MB";

    }

    private void findAllViews() {
        vid = (EditText) findViewById(R.id.vid);
        akId = (EditText) findViewById(R.id.akId);
        akSecret = (EditText) findViewById(R.id.akSecret);
        scuToken = (EditText) findViewById(R.id.scuToken);

        if(!BuildConfig.DEBUG){
            findViewById(R.id.sts_view).setVisibility(GONE);
        }

        prepareBtn = (Button) findViewById(R.id.prepare);
        downloadBtn = (Button) findViewById(R.id.download);

        wrapCheckGroup = (WrapCheckGroup) findViewById(R.id.quality_list);
        wrapCheckGroup.removeCheckBox();

        prepareBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                wrapCheckGroup.removeCheckBox();

                prepareDownloadBySts();

            }
        });

        downloadBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                downloadItem();
            }
        });

    }

    private void downloadItem() {
        CheckBox checkedItem = wrapCheckGroup.getSelectedBox();
        if (checkedItem == null) {
            Toast.makeText(getContext().getApplicationContext(), R.string.choose_a_definition_to_download, Toast.LENGTH_LONG).show();
            return;
        }
        final AliyunDownloadMediaInfo info = (AliyunDownloadMediaInfo) checkedItem.getTag();
        if (onViewClickListener != null) {
            onViewClickListener.onDownload(info);
        }

    }


    private boolean inRequest = false;

    private void prepareDownloadBySts() {
        final String mVid = vid.getText().toString();

        final String acIdStr = akId.getText().toString();
        final String akSceretStr = akSecret.getText().toString();
        final String securityToken = scuToken.getText().toString();


        if (TextUtils.isEmpty(mVid) || TextUtils.isEmpty(acIdStr) ||
                TextUtils.isEmpty(akSceretStr) || TextUtils.isEmpty(securityToken)) {

            if (inRequest) {
                return;
            }
            inRequest = true;
            VidStsUtil.getVidSts(new MyStsListener(this));

        } else {
            AliyunVidSts adb = new AliyunVidSts();
            adb.setVid(mVid);
            adb.setAcId(acIdStr);
            adb.setAkSceret(akSceretStr);
            adb.setSecurityToken(securityToken);

            downloadManager.prepareDownloadMedia(adb);
        }


//        downloadManager.prepareDownloadMedia(DemoDownloadActivity.getVidSource());

    }

    private static class MyStsListener implements VidStsUtil.OnStsResultListener{

        private WeakReference<AddDownloadView> weakView;

        public MyStsListener(AddDownloadView view)
        {
            weakView = new WeakReference<AddDownloadView>(view);
        }

        @Override
        public void onSuccess(String akid, String akSecret, String token) {
            AddDownloadView addDownloadView = weakView.get();
            if(addDownloadView != null){
                addDownloadView.onStsSuccess(akid,akSecret,token);
            }
        }

        @Override
        public void onFail() {
            AddDownloadView addDownloadView = weakView.get();
            if(addDownloadView != null){
                addDownloadView.onStsFail();
            }
        }
    }

    private void onStsFail() {

        Toast.makeText(AddDownloadView.this.getContext().getApplicationContext(), R.string.request_vidsts_fail, Toast.LENGTH_LONG).show();

        inRequest = false;
    }

    private void onStsSuccess(String akid, String akSecret, String token) {

        final String mVid = vid.getText().toString();
        AliyunVidSts adb = new AliyunVidSts();
        adb.setVid(mVid);
        adb.setAcId(akid);
        adb.setAkSceret(akSecret);
        adb.setSecurityToken(token);

        downloadManager.prepareDownloadMedia(adb);
        inRequest = false;
    }


    private OnViewClickListener onViewClickListener;

    public void setOnViewClickListener(OnViewClickListener l) {
        onViewClickListener = l;
    }

    public interface OnViewClickListener {
        void onCancel();

        void onDownload(AliyunDownloadMediaInfo info);
    }

}
