package com.aliyun.vodplayer.demo.activity.advance.liveanswer.request;

import android.content.Context;
import android.text.TextUtils;

import com.alivc.player.AliyunErrorCode;
import com.alivc.player.VcPlayerLog;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.Contants;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.QuestionResponse;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.Result;
import com.aliyun.vodplayer.utils.BaseRequest;
import com.aliyun.vodplayer.utils.HttpClientUtil;
import com.google.gson.Gson;

import org.json.JSONException;
import org.json.JSONObject;

/*
 * Copyright (C) 2010-2017 Alibaba Group Holding Limited.
 */
public class GetQuestionRequest extends BaseRequest {


    private static final String TAG = GetQuestionRequest.class.getSimpleName();

    private GetQuestionParam getQuestionParam;

    public static class GetQuestionParam {
        public String liveId;
        public String questionId;
    }

    public GetQuestionRequest(Context context, GetQuestionParam param, OnRequestListener l) {
        super(context, l);
        getQuestionParam = param;
    }


    @Override
    public void runInBackground() {

        String url = Contants.ROOM_HOST + "/app/getQuestion";

        String responseStr = null;
        try {
            String data = new Gson().toJson(getQuestionParam);

            VcPlayerLog.d(TAG, "GetQuestionRequest url = " + url + " , data = " + data);

            responseStr = HttpClientUtil.doPost(url, data);
            VcPlayerLog.d(TAG, "GetQuestionRequest response : " + responseStr);

            if (TextUtils.isEmpty(responseStr)) {

                sendFailResult(AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getCode(),
                        AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getDescription(mContextWeak.get()),
                        "");
                return;
            }

            JSONObject responseJson = new JSONObject(responseStr);
            QuestionResponse questionResponse = QuestionResponse.getFromJson(responseJson);
            if (questionResponse == null || questionResponse.data == null) {
                sendFailResult(AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getCode(),
                        AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getDescription(mContextWeak.get()),
                        "");
                return;
            } else {
                Result result = questionResponse.result;
                if (result != null) {
                    if (result.code.equalsIgnoreCase("Success")) {
                        sendSuccessResult(questionResponse.data, "");
                    } else {
                        sendFailResult(AliyunErrorCode.ALIVC_ERR_REQUEST_ERROR.getCode(),
                                result.message, "");
                    }
                } else {
                    sendFailResult(AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getCode(),
                            AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getDescription(mContextWeak.get()),
                            "");
                }

            }

        } catch (JSONException e) {

            VcPlayerLog.e(TAG , "e : " + e.getMessage());
            sendFailResult(AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getCode(),
                    AliyunErrorCode.ALIVC_ERR_DATA_ERROR.getDescription(mContextWeak.get()),
                    "");

        } catch (Exception e) {
            VcPlayerLog.e(TAG , "e : " + e.getMessage());

            sendFailResult(AliyunErrorCode.ALIVC_ERR_NO_NETWORK.getCode(),
                    AliyunErrorCode.ALIVC_ERR_NO_NETWORK.getDescription(mContextWeak.get()),
                    "");
        }

    }

    @Override
    public void stopInner() {

    }
}
